(* The syntax of our calculus *)

(* Basic Term Types *)

type nonce  = 
  | Nonce of string
  | NonceStar of string

type const  = string

type var = string

type key = string

(* error type, for printing error messages + posisions *)
type error =
  | Error of (Lexing.position * Lexing.position)
  | ErTODO

(* Public Keys (generated from secret key) *)
type pkey = 
  | PKey of key

(* Verification Keys (generated from secret key) *)
type vkey = 
  | VerKey of key

(* Terms *)
type term = 
  | VNonce of error * nonce
  | VConst of error * const
  | VVar of error * var
  | VKey of error * key
  | VPKey of error * pkey
  | VVerKey of error * vkey
  | VPair of error * term * term
  | VInl of error * term
  | VInr of error * term
  | VHash of error * term
  | VSEnc of error * term * key
  | VAEnc of error * term * pkey
  | VSign of error * term * key

(* Labels *)  
type label =
  | LL 
  | LH
  | LS

(* Types *)
type ty =
  | TLabel of label
  | TPair of ty * ty
  | TOr of ty * ty
  | TSum of ty * ty
  | TKey of label * ty
  | TRefine of term * term
  | TSEnc of ty * key
  | TAEnc of ty * key

(* Destructor applications *)
type dest = 
  | DSDec of error * var * key
  | DADec of error * var * key
  | DProj1 of error * var
  | DProj2 of error * var
  | DVer of error * var * vkey

(* Processes *)
type process =
  | PZero of error
  | PNew of error * nonce * label * process
  | POut of error * term * process
  | PIn of error * var * process
  | PPar of error * process * process
  | PRepl of error * process
  | PLet of error * var * dest * process * process
  | PIf of error * term * term * process * process
  | PMatch of error * var * var * process * var * process

module VarEnv = Map.Make
  (struct
    type t = var
    let compare = String.compare
   end)


                 

(* Environment Types *)
type nonce_env = nonce -> label option
type var_env = ty VarEnv.t
type key_env = key -> ty option


(* Constraint Types *)
type constr = term * term
type constr_set = constr list
type constr_set_env = constr list * var_env
type constr_set_set =  constr_set_env list 
